// Rover R14 v0.1 with pi-top[4]
// Build instruction on:
// https://homofaciens.de/technics-robots-R14-construction_en.htm

$fn=100;

//Raspberry Pi mount point
raspi_m_l = 58;
raspi_m_w = 49;
raspi_m_shift_x = -15;
raspi_m_shift_y = 0;
raspi_m_d = 5;
raspi_m_screw_d = 2;
raspi_m_h = 6;
raspi_m_socket = 1;

//pi-top[4] socket mount holes
ptm_dis1 = 56;
ptm_dis2 = 88;
ptm_d = 5;
ptm_shift_y = -10;

//Stepper motor dimensions
sm_d = 28;
sm_h = 19;
sms_d = 5;
sms_h = 8;     //shaft height
sms_shift = 8;  //shaft shift from center
smb_w = 15;   //buckle width
smb_h = 4.5;     //buckle height
sm_mount_dis = 35;  //mount points distance
sm_mount_d = 4;     //mount points diameter

chassis_w = 105.5;
chassis_l = 155;
chassis_h = 40;
chassis_wall = 2;

servo_w = 20;
servo_l = 40;
servo_lt = 53;
servo_h1 = 26.67;
servo_h2 = 9.35;
servo_h = servo_h1 + servo_h2;
servo_m_d = 10.15;
servo_m_l = (servo_lt-servo_l)/2;

servo_cut_w = 41;
servo_cut_h = 21;
servo_mount_x = 50/2;
servo_mount_d = 10;
servo_mount_d2 = 48.76;
servo_axis_offset = 9.66;

front_axis_x = chassis_l/2-25;
front_axis_y = 27;
rear_axis_x = -chassis_l/2+22;
rear_axis_y = front_axis_y;

//Chassis mount holes grid parameters
ch_dis_x = 10;
ch_dis_y = 10;
ch_no_x = 10;
ch_no_y = 15;
ch_no_x2 = 10;
ch_no_y2 = 4;
ch_l = 4;
ch_w = 3.2;
ch_w2 = 5;

rear_axle_d = 4.5;


track_l = 37.5;
track_h = 16;
track_w = 25;
track_wall = 2.5;
track_tolerance_w = 0.6;
tow = 8; //Tracks On Wheel

M3_hole = 3.4;
M3_nut_e = 6.4;
M4_hole = 4.4;


/*
// Chain test
for(i=[0:45:315]){
  rotate([0, 0, i]){
    translate([47, 0, 0]){
      cable_chain();
    }
  }
}
rotate([0, 0, 22.5]){
  for(i=[0:90:270]){
    rotate([0, 0, i]){
      translate([25, 0, 0]){
        cable_chain();
      }
    }
  }
}
rotate([0, 0, 11.25]){
  cable_chain();
}
*/


/*
// Multiple chains in rows
cr_no_x = 2;
cr_no_y = 4;
cr_w = 37;
cr_l = 36;
for(i=[0:1:cr_no_x-1]){
  for(j=[0:1:cr_no_y-1]){
    translate([cr_w*i, cr_l*j, 0]){
      cable_chain(1);
    }
  }
}
*/


/*
translate([0, 0, 70]){
  cube(size=[10, 250, 10], center=true);
  cube(size=[170, 10, 10], center=true);
}
*/


Assembled = 0;

if(Assembled == 0){
//  chassis();
//  cable_chain(1);
//  rear_wheel();
//  rear_wheel_mount();
//  drive_wheel();
//  camera_mount();
//  ir_mount();
//  power_pole();
//  bumper(0, 38);
//  servo_mount();
//  sonar_mount();
//  light_sensor_mount();
  camera_lever();
  
}
else{
  rotate([180, 0, 0]){
    chassis();
    translate([-75.7, front_axis_x, 13*0]){
      rotate([0, 0, 90]){
        cable_chain();
      }
    }

    translate([65.7, 100, -33]){
      rotate([0, 90, 0]){
        camera_lever();
      }
    }

    translate([-71, 102.5, -60.5]){
      rotate([90, 180, 0]){
        ir_mount();
      }
    }

    translate([0, 78, 0]){
      rotate([180, 0, 270]){
        sonar_mount();
      }
    }

    translate([35, 94, 20]){
      rotate([90, 90, 0]){
        servo_mount();
      }
    }

    translate([5, -78, 0]){
      rotate([180, 270, 270]){
        light_sensor_mount();
      }
    }

    for(i=[0:1:1]){
      mirror([i, 0, 0]){
        translate([-85, rear_axis_x, 33.5]){
          rotate([90, 0, 90]){
            rear_wheel();
          }
        }

        translate([0, 10, 35+track_h/2]){
          rotate([90, 0, 0]){
    //        rear_wheel_mount();
          }
        }


        translate([-85, front_axis_x, 33.5]){
          rotate([90, 360/(8*2), 90]){
            drive_wheel();
          }
        }
      }
    }

    translate([0, ptm_shift_y, -0.5]){
      rotate([0, 180, 90]){
        pi_top();
      }
    }

    translate([0, 80.5, -74.5]){
      rotate([0, 180, 0]){
        camera_mount();
      }
    }

      
    translate([0, -117, 50]){
      rotate([180, 0, 90]){
        bumper(0, 38);
      }
    }
    translate([0, 117, 50]){
      rotate([180, 0, 270]){
        bumper(0, 38);
      }
    }

    translate([-25, 27, -15]){
      rotate([90, 0, 90]){
  //      servo();
      }
    }

    translate([0, 0, 0]){
  //    headlight();
    }
    
    
  }



}

module camera_lever(){
  cl_l = 107;
  cl_w = 6;
  cl_wall = 3;

  cl2_l = 10;
  cl2_w = 10;
  cl2_wall = 3;
  

  difference(){
    hull(){
      translate([cl2_l/2, 10, cl_wall/2]){
        cylinder(d=cl2_w, h=cl2_wall, center=true);
      }
      translate([-cl2_l/2, 10, cl2_wall/2]){
        cylinder(d=cl2_w, h=cl2_wall, center=true);
      }
    }
    translate([cl2_l/2, 10, cl2_wall/2]){
      cylinder(d=M3_hole, h=cl_wall*1.1, center=true);
    }
    translate([-cl2_l/2, 10, cl2_wall/2]){
      cylinder(d=6, h=cl_wall*1.1, center=true);
    }
  }
  
  
  difference(){
    hull(){
      translate([cl_l/2, 0, cl_wall/2]){
        cylinder(d=cl_w, h=cl_wall, center=true);
      }
      translate([-cl_l/2, 0, cl_wall/2]){
        cylinder(d=cl_w, h=cl_wall, center=true);
      }
    }
    translate([cl_l/2, 0, cl_wall/2]){
      cylinder(d=M3_hole, h=cl_wall*1.1, center=true);
    }
    translate([-cl_l/2, 0, cl_wall/2]){
      cylinder(d=M3_hole, h=cl_wall*1.1, center=true);
    }
  }
}


module power_pole(){
  pp_d = 5;
  pp_h = 150;
  pp_wall = 5;
  pp_l = 10;
  pp_w = chassis_w;
  pp_flap_h = 10;
  pp_flap_l = 25;
  
  translate([0, 0, 0]){
    difference(){
      union(){
        roundedRect([pp_w+2*pp_wall, pp_h, pp_l], 40, [1,1,0,0]);
        hull(){
          translate([(pp_w+pp_wall)/2, (pp_h-pp_flap_h)/2, 0.1]){
            cube(size=[pp_wall, pp_flap_h, 0.05], center=true);            
          }
          translate([(pp_w+pp_wall)/2, (pp_h-pp_flap_h)/2, pp_flap_l-0.1]){
            cube(size=[pp_wall, pp_flap_h, 0.05], center=true);            
          }
        }
        hull(){
          translate([-(pp_w+pp_wall)/2, (pp_h-pp_flap_h)/2, 0.1]){
            cube(size=[pp_wall, pp_flap_h, 0.05], center=true);            
          }
          translate([-(pp_w+pp_wall)/2, (pp_h-pp_flap_h)/2, pp_flap_l-0.1]){
            cube(size=[pp_wall, pp_flap_h, 0.05], center=true);            
          }
        }
        translate([0, -(pp_h/2+10), pp_l/2]){
          intersection(){
            cube(size=[pp_l*3, 20, pp_l], center=true);
            rotate([90, 0, 0]){
              cylinder(d=pp_l*2.2, h=20, center=true);
            }
          }
        }
      }
      translate([0, pp_wall, -0.1]){
        roundedRect([pp_w, pp_h, pp_l*1.1], 40, [1,1,0,0]);
      }
      //Top holees
      translate([0, -(pp_h-pp_flap_h)/2, 5]){
        rotate([90, 0, 0]){
          cylinder(d=4.5, h=pp_w+2*pp_wall*1.1, center=true);            
        }
      }
      for(i=[0:1:1]){
        mirror([i,0,0]){
          translate([7, -(pp_h-pp_flap_h)/2-10, 0]){
            cylinder(d=2, h=pp_w+2*pp_wall*1.1, center=true);            
          }
          translate([7, -(pp_h-pp_flap_h)/2-20, 0]){
            cylinder(d=2, h=pp_w+2*pp_wall*1.1, center=true);            
          }
        }
      }

      
      //Mount holes
      hull(){
        translate([0, (pp_h-pp_flap_h)/2, 5]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pp_w+2*pp_wall*1.1, center=true);            
          }
        }
        translate([0, (pp_h-pp_flap_h)/2, 5+15]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pp_w+2*pp_wall*1.1, center=true);            
          }
        }
      }
      
    }
  }
  
  
  
  
}

module light_sensor_mount(){
  sm_w = 30;
  sm_h = 30;
  sm_wall = 3;
  sm_flap = 10;
  
  translate([-sm_wall/2, 0, sm_h/2]){
    rotate([0, 90, 0]){
      difference(){
        roundedRect([sm_h, sm_w, sm_wall], 5, [1,0,1,0]);
        translate([0, 0, -0.5]){
          roundedRect([24.2, 24.2, sm_wall], 2);
          cube(size=[18,18,15], center=true);
//          cylinder(d=M3_nut_e-0.2, h=sm_wall*5, center=true, $fn=6);
        }
      }
    }
  }
  translate([(sm_flap+sm_wall)/2, 0, 0]){
    difference(){
      roundedRect([sm_flap, sm_w, sm_wall], 5, [0,1,0,1]);
      translate([0, 10, sm_wall/2]){
        cylinder(d=M3_hole, h=sm_wall*1.1, center=true);
      }
      translate([0, -10, sm_wall/2]){
        cylinder(d=M3_hole, h=sm_wall*1.1, center=true);
      }
    }
  }
  
}

module sonar_mount(){
  sm_w = 55;
  sm_h = 30;
  sm_wall = 4;
  sm_flap = 10;
  
  translate([-sm_wall/2, 0, sm_h/2]){
    rotate([0, 90, 0]){
      difference(){
        roundedRect([sm_h, sm_w, sm_wall], 5, [1,0,1,0]);
        translate([0, 0, 1.6-sm_wall/2]){
          roundedRect([24.2, 48.2, sm_wall], 2);
          cube(size=[18, 18, sm_wall*5], center=true);
//          cylinder(d=M3_nut_e-0.2, h=sm_wall*5, center=true, $fn=6);
        }
      }
    }
  }
  translate([(sm_flap+sm_wall)/2, 0, 0]){
    difference(){
      roundedRect([sm_flap, sm_w, sm_wall], 5, [0,1,0,1]);
      translate([0, 15, sm_wall/2]){
        cylinder(d=M3_hole, h=sm_wall*1.1, center=true);
      }
      translate([0, -15, sm_wall/2]){
        cylinder(d=M3_hole, h=sm_wall*1.1, center=true);
      }
    }
  }
  
}

module servo_mount(){
  s_w = 12;
  s_l = 23;

  translate([0, 40-Assembled*40, 0+Assembled*15]){
    difference(){
      translate([0, 0, -8]){
        roundedRect([40, 30, 10], 5);
      }
      translate([15, 0, 0]){
        cylinder(d=M3_hole, h=20.1, center=true);
      }    
      translate([-15, 0, 0]){
        cylinder(d=M3_hole, h=20.1, center=true);
      }    
    }
  }

  difference(){
    cube(size=[40, 6, 15], center=true);
    translate([0, 0, 4]){
      cube(size=[s_l, 6.1, 15], center=true);
    }
    translate([15, 0, 0]){
      cylinder(d=M3_hole, h=15.1, center=true);
    }    
    translate([-15, 0, 0]){
      cylinder(d=M3_hole, h=15.1, center=true);
    }    
  }
  
}

module headlight(){
  for(i=[0:1:1]){
    mirror([i, 0, 0]){
      translate([0, 96, -79]){
        LED_module();
      }
      translate([30, 96, -79]){
       LED_module();
      }
      translate([30, 96, -50]){
        LED_module();
      }
    }
  }
}

module LED_module(cut=0){
    rotate([90, 0, 0]){
      roundedRect([24.2-cut, 24.2-cut, 12], 2, [1,1,1,1]);
    }
    translate([0, 0, 0]){
      sphere(d=10, center=true);
    }
}

module pi_top(){
  hull(){
    roundedRect([134,102,0.1],10,[1,1,1,1]);
    translate([0, 0, 43]){
      roundedRect([124,92,0.1],10,[1,1,1,1]);
    }
  }
}






module bumper(cut=0, bmp_l=23){
  
  bmp_h = 20;
  bmp_s_h = 40;//Shield height
  bmp_h_shift = 10;
  bmp_w = chassis_w + 2*40;;
  bmp_r = 200;
  bmp_wall = 3;
  bmp_gap = 20;
  bmp_gap2 = bmp_gap+6;
  bmp_edge_d = 7;
  bmp_edge_x = bmp_w/2-2.7;
  bmp_edge_y = 18.0;
  
  enforcement_edge = 40;
  linkage=30;
  M3_d = 3.5;
  
  translate([0, 0, bmp_h/2]){
    translate([bmp_edge_y, bmp_edge_x, (bmp_s_h-bmp_h)/2-Assembled*(bmp_s_h-bmp_h)/2]){
      cylinder(d=bmp_edge_d, h=bmp_s_h, center=true);
    }
    translate([bmp_edge_y, -bmp_edge_x, (bmp_s_h-bmp_h)/2-Assembled*(bmp_s_h-bmp_h)/2]){
      cylinder(d=bmp_edge_d, h=bmp_s_h, center=true);
    }
    rotate([0, 0, 0]){
      difference(){
        union(){
          
          //Schield
          translate([193, 0, (bmp_s_h-bmp_h)/2-Assembled*(bmp_s_h-bmp_h)/2]){
            difference(){
              cylinder(d=bmp_r*2, h=bmp_s_h, center=true);
              translate([0, bmp_r+bmp_w/2, 0]){
                cube(size=[bmp_r*2, bmp_r*2, bmp_s_h*1.1], center=true);
              }
              translate([0, -(bmp_r+bmp_w/2), 0]){
                cube(size=[bmp_r*2, bmp_r*2, bmp_s_h*1.1], center=true);
              }
              difference(){
                translate([bmp_wall, 0, 0]){
                  cylinder(d=bmp_r*2, h=bmp_s_h*1.1, center=true);
                }
                //Mount point on shield
                difference(){
                  translate([-bmp_r-5, 0, 0]){
                    rotate([90, 0, 0]){
                      cylinder(d=bmp_s_h*1.1, h=bmp_gap, center=true);
                    }
                  }
                  translate([10-bmp_r, 0, 0]){
                    rotate([90, 0, 0]){
                      cylinder(d=M3_hole, h=bmp_gap*1.1, center=true);
                    }
                  }
                }
              }
            }
          }
          //Front connectors
          difference(){
            union(){
              translate([bmp_l/2, (bmp_gap2+bmp_wall)/2, 0]){
                cube(size=[bmp_l+bmp_wall, bmp_wall, bmp_h], center=true);
              }
              translate([bmp_l/2, -(bmp_gap2+bmp_wall)/2, 0]){
                cube(size=[bmp_l+bmp_wall, bmp_wall, bmp_h], center=true);
              }
            }
            hull(){
              translate([4, 0, 0]){
                rotate([90, 0, 0]){
                  cylinder(d=M3_hole, h=bmp_gap*2, center=true);
                }
              }
              translate([-0.6, 0, 0]){
                cube(size=[0.2, bmp_gap*2, M3_hole], center=true);
              }
            }
            
          }
          //rear wall with mount holes
          translate([bmp_l, 0, bmp_h_shift/2]){
            difference(){
              cube(size=[bmp_wall, bmp_gap2+2*bmp_wall, bmp_h+bmp_h_shift], center=true);
              hull(){
                translate([0, ch_dis_x/2, 5+bmp_h_shift/2]){
                  rotate([0, 90, 0]){
                    cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                  }
                }
                translate([0, ch_dis_x/2, bmp_h_shift/2-5]){
                  rotate([0, 90, 0]){
                    cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                  }
                }
              }
              hull(){
                translate([0, -ch_dis_x/2, 5+bmp_h_shift/2]){
                  rotate([0, 90, 0]){
                    cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                  }
                }
                translate([0, -ch_dis_x/2, bmp_h_shift/2-5]){
                  rotate([0, 90, 0]){
                    cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                  }
                }
              }
            }


          }
        }
      }
    }
  }



  
  
}
  

module drive_wheel(){
  horn_d = 41;
  horn_w1 = 12;
  horn_w2 = 6;
  horn_thickness = 3;
  horn_mount = 13.7;
  
  chain_l = track_l-track_h;
  echo(chain_l);
  echo(sin(90));
  
  radius = (chain_l/2)/sin(180/tow)+track_h/2;
  echo(radius);
  wheel_w = track_w-2*track_wall-track_tolerance_w-1;
  hub_d = 20;
  hub_h = wheel_w+5;
  hub_hole = 3.5;
  hub_hole_2 = 10;
  hub_hole_3 = 5.5;
  servo_hub_h = 4;
  servo_hub_h2 = 3;
  
  translate([0, 0, wheel_w/2]){
    difference(){
      edge1=7;
      edge2=5;
      wx1=0;
      wy1=wheel_w/2;
      wx2=radius-edge1;
      wy2=wheel_w/2;
      wx3=radius;
      wy3=wheel_w/2-edge2;
      wx4=wx3;
      wy4=-wy3;
      wx5=wx2;
      wy5=-wy2;
      wx6=0;
      wy6=-wy1;
      rotate_extrude($fn=tow){
        polygon(points=[[wx1,wy1],[wx2,wy2],[wx3,wy3],[wx4,wy4],[wx5,wy5],[wx6,wy6]]);
      }
      
      //cylinder($fn=tow, r=radius, h=wheel_w, center=true);
      x1=0;
      y1=horn_w1/2;
      x2=horn_d/2-horn_w2/2;
      y2=horn_w2/2;
      x3=horn_d/2-horn_w2/2;
      y3=-horn_w2/2;
      x4=0;
      y4=-horn_w1/2;
      
      translate([0, 0, wheel_w/2-horn_thickness/2+0.1]){
        cylinder(d=15, h=horn_thickness, center=true);
      }
      for(i=[0:90:360]){
        rotate([0, 0, i]){
          translate([horn_mount, 0, 0]){
            cylinder(d=3, h=wheel_w*1.1, center=true);
          }
          translate([(horn_d-horn_w2)/2, 0, wheel_w/2-horn_thickness/2+0.1]){
            cylinder(d=horn_w2, h=horn_thickness, center=true);
          }
          translate([0, 0, wheel_w/2-horn_thickness+0.1]){
            linear_extrude(height=horn_thickness){
              polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
            }
          }
        }
      }
      for(i=[0:1:tow-1]){
        rotate([0, 0, 180/tow + 360/tow*i]){
          translate([radius*cos(180/tow)+0.1, 0, 0]){
            rotate([90, 0, 270]){
              linear_extrude(height=track_wall+0.1, scale=0.8){
                square(size=[15.5, hub_h*2], center=true);
              }
            }
          }
        }
      }
      cylinder(d=hub_hole, h=hub_h*1.5, center=true);
      cylinder(d=hub_hole_2, h=hub_h, center=true);
    }
  }

}


module rear_wheel_mount(){
  mount_h = 16;
  mount_h2 = 3;
  mount_d = 30;
  mount_d2=18;
  axis_d = 4;
  M4_nut_d = 8;
  M4_nut_h = 5;
  
  translate([0, 0, mount_h/2]){
    difference(){
      cylinder(d=mount_d, h=mount_h, center=true);
      cylinder(d=axis_d, h=mount_h*1.1, center=true);
      translate([0, 0, (mount_h-M4_nut_h)/2+0.1]){
        cylinder($fn=6, d=M4_nut_d, h=M4_nut_h, center=true);
      }
    }
  }
  translate([mount_d+5, 0, mount_h2/2]){
    difference(){
      cylinder(d=mount_d, h=mount_h2, center=true);
      cylinder(d=axis_d, h=mount_h*1.1, center=true);
      translate([0, mount_d/2, 0]){
        cube(size=[mount_d, mount_d-mount_d2, mount_h2*1.1], center=true);
      }
      translate([0, -mount_d/2, 0]){
        cube(size=[mount_d, mount_d-mount_d2, mount_h2*1.1], center=true);
      }
    }
  }
}

module rear_wheel(){
  chain_l = track_l-track_h;
  echo(chain_l);
  echo(sin(90));

//  radius = (chain_l/2)/sin(180/tow)+track_h/2-track_wall;
  radius = (chain_l/2)/sin(180/tow)+track_h/2-track_wall;
  echo(radius);
  wheel_w = track_w-2*track_wall-track_tolerance_w-1;
  hub_hole = 4.4;
  
  translate([0, 0, wheel_w/2]){
    difference(){
      edge1=7;
      edge2=4;
      wx1=0;
      wy1=wheel_w/2;
      wx2=radius-edge1;
      wy2=wheel_w/2;
      wx3=radius;
      wy3=wheel_w/2-edge2;
      wx4=wx3;
      wy4=-wy3;
      wx5=wx2;
      wy5=-wy2;
      wx6=0;
      wy6=-wy1;
      rotate_extrude($fn=200){
        polygon(points=[[wx1,wy1],[wx2,wy2],[wx3,wy3],[wx4,wy4],[wx5,wy5],[wx6,wy6]]);
      }
      //cylinder($fn=200, r=radius, h=wheel_w, center=true);
      cylinder(d=hub_hole, h=wheel_w*1.5, center=true);
    }
  }
}


module cable_chain(brim = 0){
  width=track_w;
  height=track_h;
  length=track_l;
  wall=track_wall;
  tolerance=0.6;
  tolerance_w=track_tolerance_w;
  tolerance_stop = 0.4;
  overlap=0.5;
  link_w=10;
  l_stop=length/2-(link_w/2-(length/2-height))-tolerance_stop;
  
  
  l=length-height;
  
  scale_a = height/(cos(180/8)*height);
  
  x1=l/2;
  y1=0;
  x2=l/2;
  y2=height/2;
  x3=l_stop;
  y3=height/2;
  x4=l_stop;
  y4=height/3;
  
  brim_h = 0.2;
  brim_d = 6;
  
  // Brim
  if(brim == 1){
    for(i=[0:1:1]){
      mirror([0, i, 0]){
        translate([track_l/3.5+2, (track_w-track_wall)/2, brim_h/2]){
          cylinder(d=brim_d, h=brim_h, center=true);
        }
        translate([-track_l/2.7-2, track_w/2+track_wall*0.7, brim_h/2]){
          cylinder(d=brim_d, h=brim_h, center=true);
        }
      }
    }
  }
  
  for(i=[0:1:1]){
    mirror([0, i, 0]){
      translate([0, -(width-wall)/2, height/2]){
        rotate([90, 0, 0]){
          difference(){
            union(){
              translate([(l/2-overlap)/2, 0, 0]){
                cube(size=[l/2+overlap, height, wall], center=true);
              }
              //stopper
              translate([l_stop/2, -(height)/4, 0]){
                //cube(size=[l_stop, height/2, wall], center=true);
              }
              translate([-(l/2-overlap)/2, 0, wall+tolerance_w]){
                cube(size=[l/2+overlap, height, wall], center=true);
              }            
              translate([0, -height/2, -wall/2]){
                linear_extrude(height=wall){
                  polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
                }
              }
              mirror([0,1,0]){
                translate([0, -height/2, -wall/2]){
                  linear_extrude(height=wall){
                    polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
                  }
                }
              }              
              translate([-l/2, 0, wall+tolerance_w]){
                scale([1, scale_a]){
                  rotate([0, 0, 360/8/2]){
                    cylinder($fn=8, d=height, h=wall, center=true);
                  }
                }
              }
              translate([l/2, 0, wall]){
                cylinder(d1=height/2, d2=height/2-wall, h=wall, center=true);
              }
            }
            translate([-l/2, 0, wall+tolerance_w]){
              cylinder(d1=height/2, d2=height/2-wall, h=wall+0.025, center=true);
            }
            translate([0, -height/2-1.7, 5]){
              rotate([70, 0, 0]){
//                cube(size=[50, 5, 5], center=true);
              }
            }
          }
        }
      }
    }
  }

    //Center bottom plate  
    difference(){
//      translate([0, 0, wall/2]){
//        cube(size=[link_w, width-2*wall, wall], center=true);
//      }
      linear_extrude(height=wall, scale=[0.7, 1]){
        square(size=[link_w, width-2*wall], center=true);
        
      }
      translate([-l/2, 0, height/2]){
        rotate([90, 0, 0]){
          cylinder(d=height+tolerance, h=width, center=true);
        }
      }
    }
    difference(){
      translate([-link_w/4+overlap/2, (width-wall+tolerance_w)/2, height/2]){
        cube(size=[link_w/2+overlap, wall+tolerance_w, height], center=true);      
      }
      translate([-l/2, (width-wall)/2, height/2]){
        rotate([90, 0, 0]){
          cylinder(d=height+tolerance, h=wall*2.1, center=true);
        }
      }
    }
    difference(){
      translate([-link_w/4+overlap/2, -(width-wall+tolerance_w)/2, height/2]){
        cube(size=[link_w/2+overlap, wall+tolerance_w, height], center=true);      
      }
      translate([-l/2, -(width-wall)/2, height/2]){
        rotate([90, 0, 0]){
          cylinder(d=height+tolerance, h=wall*2.1, center=true);
        }
      }
    }
//  }
  
}



module servo(tolerance=0, mount_holes=1){
  mh_dis1 = 49;
  mh_dis2 = 10;
  
  translate([0, 0, servo_h/2]){
    cube(size=[servo_l+tolerance, servo_w+tolerance, servo_h+tolerance], center=true);
    translate([servo_l/2-servo_axis_offset, 0, 25]){
      rotate([0, 0, 90]){
        cylinder(d=30, h=2, center=true);
      }
    }
  }
  translate([0, 0, servo_h1+1]){
    cube(size=[servo_lt, servo_w+tolerance, 3], center=true);
  }
  if(mount_holes == 1){
    translate([-servo_l/2, 0, servo_h/2]){
      cylinder(d=7, h=servo_h, center=true);
    }
    translate([mh_dis1/2, mh_dis2/2, servo_h1+1]){
      cylinder(d=M3_hole, h=20, center=true);
    }
    translate([-mh_dis1/2, mh_dis2/2, servo_h1+1]){
      cylinder(d=M3_hole, h=20, center=true);
    }
    translate([-mh_dis1/2, -mh_dis2/2, servo_h1+1]){
      cylinder(d=M3_hole, h=20, center=true);
    }
    translate([mh_dis1/2, -mh_dis2/2, servo_h1+1]){
      cylinder(d=M3_hole, h=20, center=true);
    }
  }

  
}


module ir_mount(){
  ir_l = 30;
  ir_w = 25;
  ir_h = 12;
  ir_wall = 3;
  ir_depth = 10;
  ir_space = 10;
  
  diode_d1 = 5.2;
  diode_d2 = 7.2;
  
  rec_w = 6.5;
  rec_w2 = 8;
  rec_h = 10;
  rec_d = 5;
  
  
  translate([0, 0, ir_depth/2]){
    difference(){
      cube(size=[ir_w, ir_h, ir_depth], center=true);
      // Cut IR receiver
      translate([-ir_space/2, 0, 2]){
        cube(size=[rec_w, rec_h, ir_depth], center=true);
      }
      translate([-ir_space/2, rec_h/2-rec_h/6, 2]){
        cube(size=[rec_w2, rec_h/3, ir_depth], center=true);
      }
      translate([-ir_space/2, -1, 0]){
        cylinder(d=diode_d1, h=ir_depth*1.1, center=true);
      }
      // Cut IR diode
      translate([ir_space/2, 0, 0]){
        cylinder(d=diode_d1, h=ir_depth*1.1, center=true);
      }
      translate([ir_space/2, 0, 8]){
        cylinder(d=diode_d2, h=ir_depth, center=true);
      }
    }
  }
  translate([-ir_w/2, 0, ir_l/2]){
    difference(){
      union(){
        cube(size=[ir_wall, ir_h, ir_l], center=true);
        translate([0, 0, ir_l/2]){
          rotate([0, 90, 0]){
            cylinder(d=ir_h, h=ir_wall, center=true);
          }
        }
      }
      translate([0, 0, ir_l/2]){
        rotate([0, 90, 0]){
          cylinder(d=M3_hole, h=ir_wall*1.1, center=true);
        }
      }
    }
  }
  
}


module camera_mount(){

  cam_w = 31;
  cam_h = 25;
  cam_l = 20;
  cam_pos = -3;
  cam_slot = 2.5;
  cam_slot_w = 2;
  cam_d = 32;

  cm_wall = 2;
  cm_w = cam_w + 65;
  cm_h = cam_h+cm_wall;
  cm_h2 = cm_h + 37;
  lever_l = 6;
  lever_radius = 5;
  lever_z = cm_h/2-lever_radius;

  lever_l = 10;
  lever_radius = 5;
  lever_gap = cm_w-cm_wall;
  lever_z = lever_radius-cm_h/2;

  // Servo lever
  lever2_l = -10;
  lever2_radius = 5;
  lever2_z = 0;

  LED_d = 5.1;
  LED_w = cm_w - 29 - 5;
  LED_no = 1;

  mount_l = 10;
  mount_h = 87;
  mount_edge = 40;
  mount_shift = 0;
  
  mount_d = 10;
  
  left_shift=2.5*0;
  
  // Front plate mount holes
  fp_dis = 55;
  

  //Mount plates
  for(i=[0:1:1]){
    mirror([i, 0, 0]){
      translate([(-chassis_w-cm_wall*2)/2-40+38*Assembled, -9.5-19+19*Assembled, -81.5*Assembled]){
        rotate([0, 270*Assembled, 180*Assembled]){
          // Mount point servo plugs
          sp_h = 10;
          sp_w = 17;
          sp_l = 25;
          if(i==1){
            translate([mount_h/2-3, (sp_w-mount_d)/2, 0]){
              difference(){
                roundedRect([sp_l, sp_w, sp_h], 2,[1,1,1,1]);
                translate([0, 1, 4+0.1]){
                  linear_extrude(height=6, scale=[0.7, 1]){
                    square(size=[sp_l-2, sp_w], center=true);
                  }
                }
              }
            }
          }

          difference(){
            hull(){
              translate([0, mount_l, cm_wall]){
                cylinder(d=mount_d, h=cm_wall*2, center=true);
              }
              translate([0, 0, cm_wall]){
                cylinder(d=mount_d, h=cm_wall*2, center=true);
              }
              translate([mount_h, 0, cm_wall]){
                cylinder(d=mount_d, h=cm_wall*2, center=true);
              }
            }
            translate([0, mount_l, cm_wall]){
              cylinder(d=M3_hole, h=cm_wall*2.1, center=true);
            }
            translate([0, 0, cm_wall]){
              cylinder(d=M3_hole, h=cm_wall*2.1, center=true);
            }
            translate([mount_h, 0, cm_wall]){
              cylinder(d=M3_hole, h=cm_wall*2.1, center=true);
            }
          }
        }
      }
    }
  }



  // Front mount plate
  translate([0, 13+77*(1-Assembled), 0]) rotate([90*(1-Assembled), 0, 0])
  difference(){
    translate([0, 0, cm_h/2]){
      rotate([90, 0, 180]){
        difference(){
          translate([0, (cm_h2-cm_h)/2, 0]){
            roundedRect([cm_w, cm_h2, cm_wall*2], 3);
          }
          
          //LED cuts
          translate([0, 37, 10]){
            rotate([90, 0, 0]){
              LED_module(5);
            }
          }
          for(i=[0:1:1]){
            mirror([i, 0, 0]){
              translate([LED_w/2, 0, 10]){
                rotate([90, 0, 0]){
                  LED_module(5);
                }
              }
              translate([LED_w/2, 37, 10]){
                rotate([90, 0, 0]){
                  LED_module(5);
                }
              }
            }
          }
        }
      }
    }

    // Camera box
    rotate([90, 0, 180]){
      translate([-left_shift, cam_d/2+1, -10]){
        roundedRect([cam_w+2*cm_wall+2, cm_h+10, cam_l], 2, [0,0,1,1]);
        rotate([0, 0, 90]){
//          cylinder(d=cam_d, h=20*cm_wall, center=true);
        }
      }
    }
    // Mount holes
    translate([fp_dis/2, 0, cm_h2/2]){
      rotate([90, 0, 0]){
        cylinder(d=M3_hole, h=cm_wall*40, center=true);
      }
    }
    translate([-fp_dis/2, 0, cm_h2/2]){
      rotate([90, 0, 0]){
        cylinder(d=M3_hole, h=cm_wall*40, center=true);
      }
    }
    // Servo lever cut
    translate([cm_wall-cm_w/2, 0, lever2_radius]){
      cube(size=[cm_wall*2+1, 10, lever2_radius*2+1], center=true);
    }
  }

  // Main mount plate
  translate([0, 0, 0]){
    //Camera box
    difference(){
      translate([0, 0, cm_h/2]){
        rotate([90, 0, 180]){
          difference(){
            union(){
              translate([0, (cm_h2-cm_h)/2, 0]){
                roundedRect(size=[cm_w, cm_h2, cm_wall*2], radius=3, edges=[0,0,1,1]);
              }
              //Camera box
              translate([-left_shift, (cam_d-cam_h)/2, cm_wall]){
                roundedRect(size=[cam_w+2*cm_wall, cam_d+2, cam_l], radius=2);
              }
            }
            //Camera cable slit
            translate([0, 4.5, cm_wall/2]){
//              cube(size=[15, 30, 10], center=true);
            }
            
            //LED cuts
            translate([0, 37, 3.4-cm_wall]){
//              cube(size=[18, 18, cm_wall*5], center=true);
              translate([0, 0, 13]){
                rotate([90, 0, 0]){
                  LED_module();
                }
              }              
            }
            for(i=[0:1:1]){
              mirror([i, 0, 0]){
                translate([LED_w/2, 0, 3.4-cm_wall]){
//                  cube(size=[18, 18, cm_wall*5], center=true);
                  translate([0, 0, 13]){
                    rotate([90, 0, 0]){
                      LED_module();
                    }
                  }              
                }
                translate([LED_w/2, 37, 3.4-cm_wall]){
//                  cube(size=[18, 18, cm_wall*5], center=true);
//                  cylinder(d=M3_nut_e-0.2, h=cm_wall*5, center=true, $fn=6);
                  translate([0, 0, 13]){
                    rotate([90, 0, 0]){
                      LED_module();
                    }
                  }              
                }
              }
            }
          }
        }
      }
      // Cut camera
      translate([left_shift, cam_l/2+cm_wall*2-2, cm_h/2+cm_wall]){
//        cube(size=[cam_w-2*cam_slot_w, cam_l-4, cam_h], center=true);
      }
      translate([left_shift, cam_l/2+cm_wall*2, cm_h/2+cm_wall+12.5]){
//        cube(size=[15, cam_l, cam_h], center=true);
      }
      translate([left_shift, cam_l/2+cm_wall*2, cam_d/2+1]){
        rotate([90, 0, 0]){
          cylinder(d=cam_d, h=cam_l*2, center=true);
        }
      }
      // Cut camera slit
      translate([left_shift, cam_l/2+cam_pos+cm_wall, cm_h/2+cm_wall]){
//        cube(size=[cam_w, cam_slot, cam_h], center=true);
      }
      // Mount holes front plate
      translate([fp_dis/2, 0, cm_h2/2]){
        rotate([90, 30, 0]){
          cylinder(d=M3_hole, h=cm_wall*4.1, center=true);
          translate([0, 0, 2]){
            cylinder(d=M3_nut_e, h=cm_wall*4.1, center=true, $fn=6);
          }
        }
      }
      translate([-fp_dis/2, 0, cm_h2/2]){
        rotate([90, 30, 0]){
          translate([0, 0, 2]){
            cylinder(d=M3_nut_e, h=cm_wall*4.1, center=true, $fn=6);
          }
          cylinder(d=M3_hole, h=cm_wall*4.1, center=true);
        }
      }
    }
    
    
    //Lever mounts
    translate([left_shift, 0, 0]){
      for(i=[0:1:1]){
        mirror([i, 0, 0]){
          translate([lever_gap/2, 0, cm_h/2]){
            rotate([0, 90, 0]){
              difference(){
                hull(){
                  translate([-lever_z, -lever_l, 0]){
                    cylinder(r=lever_radius, h=cm_wall, center=true);  
                  }
                  translate([0, cm_wall/2, 0]){
                    cube(size=[cm_h, cm_wall, cm_wall], center=true);
                  }
                }
                translate([-lever_z, -lever_l, 0]){
                  cylinder(d=M3_hole, h=cm_wall*1.1, center=true);  
                }          
              }
            }
          }
        }
      }
    }

    //Lever servo
    translate([cm_wall-cm_w/2, 0, 0]){
      rotate([0, 90, 0]){
        difference(){
          union(){
            hull(){
              translate([lever2_z-lever2_radius, -lever2_l, 0]){
                cylinder(r=lever2_radius, h=cm_wall*2, center=true);  
              }
              translate([-lever2_radius, cm_wall/2, 0]){
                cube(size=[lever2_radius*2,cm_wall,cm_wall*2],center=true);
              }
            }
          }
          translate([lever2_z-lever2_radius, -lever2_l, 0]){
            cylinder(d=M3_hole, h=cm_wall*2.1, center=true);  
          }          
        }
      }
    }
    
  }

}



module chassis(){
//  rotate([90, 0, 270])servo();
  
  translate([0, 0, chassis_wall/2]){
    difference(){
      union(){
        cube(size=[chassis_w, chassis_l, chassis_wall], center=true);
        translate([(chassis_w-chassis_wall)/2, 0, chassis_h/2]){
          cube(size=[chassis_wall, chassis_l-2*chassis_wall, chassis_h], center=true);
        }
        translate([-(chassis_w-chassis_wall)/2, 0, chassis_h/2]){
          cube(size=[chassis_wall, chassis_l-2*chassis_wall, chassis_h], center=true);
        }
        translate([0, (chassis_l-chassis_wall)/2, chassis_h/2]){
          cube(size=[chassis_w, chassis_wall, chassis_h], center=true);
        }
        translate([0, -(chassis_l-chassis_wall)/2, chassis_h/2]){
          cube(size=[chassis_w, chassis_wall, chassis_h], center=true);
        }
      }
      
      // Front motor cuts
      for(i=[0:1:1]){
        mirror([i, 0, 0]){
          translate([20-chassis_w/2, front_axis_x-servo_axis_offset, front_axis_y]){
            rotate([90, 0, 270]){
              servo(1);
            }
          }
        }
      }

      //Rear axis cuts
      //diff=(track_l-track_h)/2;
      diff=15;
      translate([0, rear_axis_x-diff/2, rear_axis_y]){
        rotate([90, 0, 90]){
          cylinder(d=rear_axle_d, h=chassis_w*1.1, center=true);
        }
      }
      translate([0, rear_axis_x+diff/2, rear_axis_y]){
        rotate([90, 0, 90]){
          cylinder(d=rear_axle_d, h=chassis_w*1.1, center=true);
        }
      }
      translate([0, rear_axis_x, rear_axis_y]){
        cube(size=[chassis_w*1.1, diff, rear_axle_d], center=true);
      }

      //Servo plug hole
      translate([0, 66, 0]){
        cylinder(d=15, h=5, center=true);
      }
      
      // pi-top[4] mount holes
      translate([ptm_dis1/2, ptm_shift_y+ptm_dis2/2, chassis_wall/2*0]){
        cylinder(d=ptm_d, h=chassis_wall*1.1, center=true);
      }
      translate([-ptm_dis1/2, ptm_shift_y+ptm_dis2/2, chassis_wall/2*0]){
        cylinder(d=ptm_d, h=chassis_wall*1.1, center=true);
      }
      translate([-ptm_dis1/2, ptm_shift_y-ptm_dis2/2, chassis_wall/2*0]){
        cylinder(d=ptm_d, h=chassis_wall*1.1, center=true);
      }
      translate([ptm_dis1/2, ptm_shift_y-ptm_dis2/2, chassis_wall/2*0]){
        cylinder(d=ptm_d, h=chassis_wall*1.1, center=true);
      }

      //mount holes grid bottom
      for(j=[0:1:ch_no_y-1]){
        edge_x = (chassis_w - (ch_no_x-1) * ch_dis_x)/2;
        edge_y = (chassis_l - (ch_no_y-1) * ch_dis_y)/2;
        for(i=[0:1:ch_no_x-1]){
          a=-chassis_w/2 + edge_x + ch_dis_x * i;
          b=-(chassis_l+chassis_wall)/2 + edge_y + ch_dis_y * j;
          translate([a, b, 0]){
            if(!(j==1 && i==2)&&
              !(j==2 && i==2)&&
              !(j==1 && i==7)&&
              !(j==2 && i==7)&&
              !(j==13 && i==4)&&
              !(j==13 && i==5)&&
              !(j==14 && i==4)&&
              !(j==14 && i==5)&&
              !(j==10 && i==2)&&
              !(j==11 && i==2)&&
              !(j==10 && i==7)&&
              !(j==11 && i==7)
              ){
              translate([0, ch_l/2, 0]){
                cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                translate([0, 0, 0.75]){
                  cylinder(d=M3_nut_e, h=chassis_wall, center=true, $fn=6);
                }
              }
            }
          }
        }
      }

      //mount holes grid left and right
      translate([0, 0, chassis_h/2]){
        rotate([0, 90, 0]){
          edge_y = (chassis_l - (ch_no_y-1) * ch_dis_y)/2;
          edge_z = (chassis_h - (ch_no_y2-1) * ch_dis_y)/2+chassis_wall/2;
          for(i=[0:1:ch_no_y-1]){
            a=-(chassis_l+chassis_wall)/2 + edge_y + ch_dis_y * i;
            b=(chassis_h)/2 - edge_z;
            translate([b, a, 0]){
              translate([0, ch_l/2, 0]){
                cylinder(d=M3_hole, h=chassis_w*1.1, center=true);
                cylinder(d=M3_nut_e, h=chassis_w-1.95, center=true, $fn=6);
              }
              if(i<9){
                translate([11-chassis_h, ch_l/2, 0]){
                  cylinder(d=M3_hole, h=chassis_w*1.1, center=true);
                  cylinder(d=M3_nut_e, h=chassis_w-1.95, center=true, $fn=6);
                }
              }
            }
          }
        }
      }
      
      
      //mount holes grid front and rear
      translate([0, 0, chassis_h/2]){
        rotate([90, 0, 0]){
          edge_x = (chassis_w - (ch_no_x2-1) * ch_dis_x)/2;
          edge_y = (chassis_h - (ch_no_y2-1) * ch_dis_y)/2;
          for(j=[0:1:ch_no_y2-1]){
            for(i=[0:1:ch_no_x2-1]){
              a=-chassis_w/2 + edge_x + ch_dis_x * i;
              b=-(chassis_h+chassis_wall)/2 + edge_y + ch_dis_y * j;
              translate([a, b, 0]){
                translate([0, ch_l/2, 0]){
                  cylinder(d=M3_hole, h=chassis_l*1.1, center=true);
                  cylinder(d=M3_nut_e, h=chassis_l-1.95, center=true, $fn=6);
                }
              }
            }
          }
        }
      }

    }
  }
   
  if(Assembled == 0){
    //bridge_post(10, width=6, brim_w=15, brim_l=6, link=0);
    for(i=[0:1:1]){
      mirror([i, 0, 0]){
        translate([-59.0, 50, 0]){
          rotate([0, 0, 90]){
            bridge_post(38.55, 8.3, 17, 10, 0);
          }
        }
        translate([-59, 33, 0]){
          rotate([0, 0, 90]){
            bridge_post(38.55, 8.3, 17, 10, 0);
          }
        }
        translate([-58, 43, 15]){
          cube(size=[7, 20, 1], center=true);
        }
      }
    }
  }
}



module roundedRect(size, radius, edges=[1, 1, 1, 1]){
  x = size[0];
  y = size[1];
  z = size[2];

  linear_extrude(height=z){
    hull(){
        // place 4 circles in the corners, with the given radius
        if(edges[0]){
          translate([(-x/2)+(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
        if(edges[1]){
          translate([(x/2)-(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[2]){
          translate([(-x/2)+(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[3]){
          translate([(x/2)-(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
    }
  }
}

module bridge_post(height, width=6, brim_w=10, brim_l=6, link=0){
  edge=6;
  tip=edge;
  tip_w = 0.2;
  brim_h = 0.3;
  socket_h = chassis_wall;
  
  x1=edge/2;
  y1=0;
  x2=edge/2;
  y2=height-tip;
  x3=-width;
  y3=height-tip;
  x4=-edge/2;
  y4=socket_h;
  x5=-edge/2;
  y5=0;
  
  translate([edge/2, 0, brim_h/2]){
    cube(size=[brim_w, brim_l, brim_h], center=true);
  }
  
  rotate([90, 0, 90]){
    linear_extrude(height=edge){
      polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5]]);
    }
  }
  
  if(link>0){
    translate([link/2, 0, height/2]){
      cube(size=[link, edge/2, 2], center=true);
    }
  }
  
  x1t=edge/2;
  y1t=0;
  x2t=0;
  y2t=edge;
  x3t=-edge/2;
  y3t=0;
  
  translate([edge/2, edge/2, height-tip]){
    rotate([90, 0, 0]){
      linear_extrude(height=edge/2+width){
        polygon(points=[[x1t,y1t],[x2t,y2t],[x3t,y3t]]);
      }
    }    
  }

  translate([edge/2, -(width-edge/2)/2, height-tip/2]){
    cube(size=[tip_w, width+edge/2, tip], center=true);    
  }

}
